@@ -597,6 +597,97 @@
    deallocate(rnn,fnn)
    end subroutine calc_diff_exp2
    
+  subroutine calc_diff_exp2_3D(isdiff,iord,n,dnr,xh,rn,fn,dfn,ista_nrc,iend_nrc,ddfn)
+    implicit none
+    integer,intent(in)  :: isdiff,iord, n, dnr
+    real(8),intent(in)  :: xh,rn(n), fn(ista_nrc-6*dnr:iend_nrc+6*dnr)
+    real(8),intent(out) :: dfn(ista_nrc:iend_nrc)
+    integer, intent(in) :: ista_nrc, iend_nrc
+    real(8),optional,intent(inout):: ddfn(ista_nrc:iend_nrc)
+    real(8) :: r, f, df,ddf,h
+    real(8),allocatable :: p(:,:), dp(:,:), ddp(:,:)
+    integer :: n1, n2, i, pdim, indx
+    integer       :: id_sname = -1
+
+    pdim = 1+2*iord
+    h = 1.d0/xh
+    allocate(p(pdim,pdim),dp(pdim,pdim))
+    if(isdiff >=2)  allocate(ddp(pdim,pdim))
+
+!!$    do i = 1,n
+    do i = max(1,ista_nrc),min(iend_nrc,n)
+       r = rn(i)
+       if (i < iord+1) then
+          n1 = 1 ; n2 = 1 + 2*iord
+       else if (i > n-iord) then
+          n1 = n - 2*iord ; n2 = n
+       else
+          n1 = i - iord ; n2 = i + iord
+       end if
+       indx = i
+      ! indx-n1 = i-(i-iord) = iord
+!!$       call diff_exp2(isdiff,n1,n2,rn,fn,r,df,ddf)
+       call diff_exp2(isdiff,n1,n2,fn,r,df,ddf)  ! isdiff,n1,n2,fn,r -> df,ddf
+!!$      call diff_exp(ier,isdiff,n1,n2,rn,fn,r,f,df,ddf)
+       dfn(i) = df
+       if(isdiff==2) ddfn(i) = ddf
+    end do
+    if(isdiff >=2) deallocate(ddp)
+    deallocate(p,dp)
+  contains
+    subroutine diff_exp2(isdiff,n1,n2,fn,r,df,ddf) ! isdiff,n1,n2,fn,r
+!---------------------------------------------------------------------
+!
+!   Program written by Masakuni Okamoto
+!  Revised for logarithmic grid by Takahiro Yamasaki, 2010/06/10 
+!
+!---------------------------------------------------------------------
+      integer,intent(in)  :: isdiff,n1,n2
+      real(8),intent(in)  :: fn(ista_nrc-6:iend_nrc+6), r
+      real(8),intent(out) :: df,ddf
+      real(8) :: denom_inv, a, b, c, x, x1, x2, dn2minusn1
+      integer :: n, i, j, n1_delta
+      n = n2-n1+1
+      p(1:n,1) = fn(n1:n2) ;  dp(:,:) = 0.d0 
+      if(isdiff>=2) ddp(:,:) = 0.d0
+      c = h*(n2-n1)
+      n1_delta =indx-n1
+      dn2minusn1 = 1.d0/dble(n2-n1)
+
+      if ((isdiff >= 0).and.(n >= 2)) then
+         do j = 2,n
+            denom_inv = -dble(n-1)/dble(j-1)
+            do i = 1,n+1-j
+               p(i,j) = ((n1_delta-i-j+2)*p(i,j-1) + (i-1-n1_delta)*p(i+1,j-1))*dn2minusn1 * denom_inv
+            end do
+         end do
+      end if
+      if (isdiff >= 1) then
+         do j = 2,n
+            denom_inv = -dble(n-1)/dble(j-1)
+            do i = 1,n+1-j
+               dp(i,j) = ((n1_delta-i-j+2)*dp(i,j-1) + (i-1-n1_delta)*dp(i+1,j-1))*dn2minusn1 &
+                    &     + (p(i,j-1)-p(i+1,j-1))
+               dp(i,j) = dp(i,j) * denom_inv
+            end do
+         end do
+      end if
+      if (isdiff >= 2) then
+         do j = 2,n
+            denom_inv = -dble(n-1)/dble(j-1)
+            do i = 1,n+1-j
+               ddp(i,j) = ((n1_delta-i-j+2)*ddp(i,j-1) + (i-1-n1_delta)*ddp(i+1,j-1))*dn2minusn1 &
+                    &      + 2.d0*(dp(i,j-1)-dp(i+1,j-1))
+               ddp(i,j) = ddp(i,j) * denom_inv
+            end do
+         end do
+      end if
+      df  = dp (1,n) * (1.d0/r/c)
+      if(isdiff >=2) ddf = ddp(1,n) * (1.d0/r/c)**2 - dp(1,n) * (1.d0/r/r/c)
+    end subroutine diff_exp2
+
+  end subroutine calc_diff_exp2_3D
+
 !=====================================================================
    subroutine calc_diff_exp3(ier,iord,n,dn,rn,fn,dfn)
 !---------------------------------------------------------------------
